/*                               S Z . C       
 *
 *   Goes through each FILES.BBS and strips off ending ^Z if there.

 *   This program was last compiled using MicroSoft C V4.0. 09/21/1987
 */

/*
 *  Modified 21 Sep 87 by Vincent T. Mills - Version 1.0
 *  Gliff Opus 119/14 (916) 891-3410
 *
 *  Kept area number limit and priv level options, but stripped out all other
 *  options that don't apply.  Simplified utility just reduces the file length
 *  of the FILES.BBS file 1 byte when there is a cntrol-Z (26) ending the file.
 *
 *  Modified 23 Jan 87 by Vincent E. Perriello  - Version 3.4
 *  Naugy-Net BBS Opus 141/491 (203) 729-7569
 *
 *  Supports longer lines (up to 255 characters) and wildcard file names.
 *  Re-enabled support for non-contiguous SystemNN.BBS files (e.g. SYSTEM1.BBS
 *  through SYSTEM15.BBS for example, then SYSTEM59.BBS) for special cases.
 *  Add support for ASSTSYSOP and HIDDEN privs. Also ported back to MicroSoft 
 *  C (V4.0) to stay. Thanks to a couple of nice routines I swiped from Wynn 
 *  it works pretty well. Use Wynn's OPUS.H structure -- I want to add more 
 *  Opus support but don't know how far I'll take it.
 *
 *  Modified 17 May 86 by Vincent E. Perriello  - Version 3.3
 *
 *  Integrated Kevin-Neal Klop code (parallel development) to support
 *  the -v switch; but since I want to do so without breaking anything
 *  I am going to leave -p, which is really equivalent to -vSYSOP.
 *  Also, add -i switch, which prints the DIR.BBS entries ONLY, as an
 *  "Index" ..., and -f switch which will pepper output with formfeeds.
 *
 *  Modified 20 Apr 86 by Vincent E. Perriello  - Version 3.2
 *
 *  Modify the -o (number) switch support to automatically override the
 *  privilege check. 
 *
 *  Modified 13 May 86 by Kevin-Neil Klop - Version 3.11
 *  AAPIS BBS Fido 103/525 (714) 731-1039
 *
 *  Replaced -p option with -v option to allow the report to output all file
 *  areas with a security level less than or equal to a user specified value
 *  This makes the utility more useful to semi-public BBSs
 *
 *  Modified 01 Mar 86 by Vincent E. Perriello  - Version 3.1
 *
 *  Add support for printing out a single directory's stuff. This is done
 *  by ignoring the absence of SYSTEM??.BBS and just trying to open DIR.BBS
 *  and FILES.BBS in the current directory. Also add -o (number) switch to
 *  only output file area (number).
 *
 *  Modified 26 Apr 85 by Robert Briggs
 *  Prog_Tech_BBS Fido 15/464 (801) 264-8290
 *
 * Modified to use FREOPEN where possible - the original program 
 * would quit functioning on the 18th file area.
 * Whether the bug was in CI-C86 or DOS 3.0 was never determined.
 * Symptoms were:
 *    
 * On the 18th file open for any given FILE variable (i.e bbs, junk, sysv),
 * the system would start returning ERROR instead of a new stream pointer.
 * On every open and close, the returned file number was one larger.
 * It LOOKED like file DCB's were not being re-used, even after a file
 * was closed.  The solution was kind a a hack - all stream pointers were
 * made global, set to NULL originally, and FREOPEN's were done if the
 * stream pointers were non NULL.  Files are only closed at the end of the
 * program, just prior to exit. 
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <dos.h>

#include "opus.h"

#define MAXLINE 255                     /* max char string space */
#define EOS 0                           /* standard end of string */
#define TRUE 1
#define FALSE 0
#define CTRL_Z      0x1a

extern   char  *fgets();
extern   char  *calloc();

extern   FILE  *fopen();
extern   FILE  *fileopen();

FILE  *bbs, *sysv;

char  fastr[4];
char  nodestr[12];

int low_file_area;
int high_file_area;
int report_level;

long  size_total, files_total;

static char *hlp1[] = 
   {
   "\n",
   "SZ: Strip control-Z's off FILES.BBS listings\n",
   "    V1.0 - 09/18/1987 - by Vincent T. Mills Opus 119/14\n",
   "Based on FILESPRN by Vincent E. Perriello, Opus 141/491\n",
   "Which is based on FILER by Robert Briggs, FIDO 15/464\n",
   ""
   };

static char *hlp2[] = 
   {
   "\n",
   "SZ V1.0 - 09/18/1987 - by Vincent T. Mills Opus 119/14\n",
   "Usage:  sz [-o (number)][-v(level)] [-p]\n",
   "Options:\n",
   "        -o (number)  only strip file area (number).\n",
   "        -v(level)    only strip areas at or below specified level\n",
   "                     levels are: TWIT, DISGRACE, NORMAL, PRIVEL,\n",
   "                     EXTRA, ASSTSYSOP, SYSOP and HIDDEN.\n",
   "                     (default level is HIDDEN for scanning all areas.)\n",
   ""
   };


/* 
 * ERROUT -- write an array of lines to stderr.
 * 
 */
errout(s)
char *s[];
{
int i;
for (i = 0 ; *s[i] ; i++)
   fputs(s[i],stderr);
}

/* 
 * usage() -- display the command set.
 * 
 */
usage ()
{
   errout(hlp2);
   exit();
}

/* 
 * fileopen(file,mode,fd) Open a file. Use fopen() if the file descriptor has 
                          not been used yet, or freopen() if it has. This 
                          seems to have the desirable side effect of making it 
                          all work with a lot less disk accesses, besides 
                          addressing the problem that Bob Briggs found. 
 * 
 */
FILE *fileopen(file,mode,fd)
char *file, *mode;
FILE *fd;
{
if (fd == NULL)               /* opened before?   */
   return (fopen(file,mode));    /* no, fopen it     */
else
   return (freopen(file,mode,fd));     /* freopen the file */
}

/* 
 *  SZfiles(path):       Reads FILES.BBS from drive and path
 *                       specified by caller.
 *                       Clears ending Control-Z's.
 * 
 */
SZfiles(path)
char *path;
{
   char  filenm[MAXLINE];

   strcpy(filenm,path);          /* get the path */
   strcat(filenm,"files.bbs");         /* create composite file name */

   /* open the file */
   /* for reading and writing */
   bbs = fileopen(filenm, "r+b",bbs);

   if (bbs == NULL)         /* couldn't open the file */
   {
      printf("Couldn't open %s for stripping\n",filenm); 
      return;              /* back to caller        */
   }

   fseek(bbs, (long) -1, SEEK_END);     /* read last char of file */

   if(fgetc(bbs) == CTRL_Z) {
      fseek(bbs, (long) -1, SEEK_END);
      chsize( fileno(bbs), (long) ftell(bbs) );
/*    fclose(bbs); */
      printf ("Stripped a Cntl-Z from %s\n", filenm);
   }
   return;                 /* back to caller          */

}  /* end of SZfiles() */

/* 
 *  main(argc,argv)
 * 
 */
main(argc,argv)
int argc;
char **argv;
{
static   char  descr[40]="Download Area ";

typedef struct _mail    MSYS;
typedef struct _sys  VSYS;
MSYS    mbbs;
VSYS    vbbs;
int   working=TRUE, i, pass;
char  *sysname[16];

#define mynode  mbbs.node
#define mynet   mbbs.net

#define dpriv  vbbs.priv
#define dpath  vbbs.filepath

/*
 * Initialize the file stream pointers and set up the defaults.
 */

files_total = size_total = 0l;         /* but initialize total=0  */
low_file_area = 1;                              /* first file area         */
high_file_area = 99;                            /* last file area (we hope)*/

report_level = HIDDEN;           /* show ALL files */

sysv = bbs = NULL;        /* initialize stream pointers */
strcpy (nodestr,"");          /* Net, null by default */

/*
 * Now tell the world exactly who you are. Use stderr so that if
 * I/O redirection is going on, it won't go in the file, and the
 * user will still get to see it.
 */

errout(hlp1);

/*
 * Parse the switches. If you find one you don't recognize, then
 * output the usage blurb and exit.
 */

for (i = 1; i < argc; i++)
   {
   if (argv[i][0] == '-')        /* switch */
      switch (toupper(argv[i][1]))  /* switch character */
         {

         case 'O':      /* only one file area       */
         if (low_file_area = atoi(argv[i+1]))
         {                  /* if we got a good number  */
            high_file_area = low_file_area;  /* copy it */
            i++;            /* and bump the arg counter */
         }
         else
            low_file_area = 1;      /* else reset to 1  */
         report_level = HIDDEN;  /* override privilege check */
         break;

         case 'V':      /* Privilege level to check */
         switch (toupper(argv[i][2]))/* now check it out     */
            {
            case 'T':
            report_level = TWIT;
            break;

            case 'D':
            report_level = DISGRACE;
            break;

            case 'N':
            report_level = NORMAL;
            break;

            case 'P':
            report_level = PRIVEL;
            break;

            case 'E':
            report_level = EXTRA;
            break;

            case 'A':
            report_level = ASSTSYSOP;
            break;

            case 'S':
            report_level = SYSOP;
            break;

            case 'H':
            report_level = HIDDEN;
            break;

            default:
            usage();
             }
             break;

         default:
         usage();    /* tell them how to do it   */
         }
   else
      usage();       /* not a switch? */
   }

/*
 * For each SYSTEMxx.BBS found, if "normal" users are permitted to 
 * view the directory (or if the -p switch has been specified), then
 * read and display the FILES.BBS a la Fido. If we don't find any
 * SYSTEMxx.BBS file, then try to display files in the default directory.
 */
for (i = low_file_area; i <= high_file_area; i++)
   {
   sprintf(sysname,"system%d.bbs",i);  /* construct filename */
      if ((sysv=fileopen(sysname,"rb",sysv)) == NULL) /* open table file, */
      {                                               /* we failed!   */
      if (i == low_file_area)    /* if this is the first one */
         {
         SZfiles("");    /* try for this directory   */
         return;        /* then go away          */
         }
      continue;         /* failure, try next one    */
      }

   if (fread(&vbbs,sizeof(VSYS),1,sysv) != 1)   /* try to read it */
      abort("SZ:  error reading %s",sysname);

   if ((dpriv>report_level)||(dpath[0]==EOS))
      continue;         /* privileged, skip this one */

   SZfiles(dpath);            /* strip ctlz */
   }
/*
 * Now close up any stream pointers still open
 */

if (sysv)
   fclose(sysv);
if (bbs)
   fclose(bbs);

exit(0);             /* back to DOS */

}  /* end of main() */

/* end of sz.c */

